
// DynamicsCont.fsi - Aivika

// Copyright (c) 2009, 2010  David Sorokin. All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 
//   * Redistributions of source code must retain the above copyright
//     notice, this list of conditions and the following disclaimer.
// 
//   * Redistributions in binary form must reproduce the above
//     copyright notice, this list of conditions and the following
//     disclaimer in the documentation and/or other materials
//     provided with the distribution.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
// FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
// THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
// OF THE POSSIBILITY OF SUCH DAMAGE.

//
// Author: David Sorokin <david.sorokin@gmail.com>
//

namespace Maritegra.Aivika

open System

type internal DynamicsContFun<'a> = Dynamics<('a -> unit) * (exn -> unit)> -> Dynamics<unit>

[<Sealed>]
type DynamicsCont<'a> =
    internal new: DynamicsContFun<'a> -> DynamicsCont<'a>
    member internal Fun: DynamicsContFun<'a>

[<AutoOpen>]
module internal DynamicsContOp =
    val inline (<|--): DynamicsCont<'a> -> DynamicsContFun<'a>

[<Sealed>]
type DynamicsContBuilder =

    member Return: 'a -> DynamicsCont<'a>
    member ReturnFrom: DynamicsCont<'a> -> DynamicsCont<'a>
    member Bind: DynamicsCont<'a> * ('a -> DynamicsCont<'b>) -> DynamicsCont<'b>
    member Delay: (unit -> DynamicsCont<'b>) -> DynamicsCont<'b>
    member Zero: unit -> DynamicsCont<unit>
    member Combine: DynamicsCont<unit> * DynamicsCont<'a> -> DynamicsCont<'a>
    member For: seq<'a> * ('a -> DynamicsCont<unit>) -> DynamicsCont<unit>
    member While: (unit -> bool) * DynamicsCont<unit> -> DynamicsCont<unit>
    member Using: 'a * ('a -> DynamicsCont<'b>) -> DynamicsCont<'b> when 'a :> IDisposable
    member TryFinally: DynamicsCont<'a> * (unit -> unit) -> DynamicsCont<'a>
    member TryWith: DynamicsCont<'a> * (exn -> DynamicsCont<'a>) -> DynamicsCont<'a>

[<AutoOpen>]
module DynamicsContWorkflow =     
    val dynamicscont: DynamicsContBuilder

[<RequireQualifiedAccess>]
[<CompilationRepresentation (CompilationRepresentationFlags.ModuleSuffix)>]
module DynamicsCont =

    [<CompiledName ("Lift")>]
    val lift: Dynamics<'a> -> DynamicsCont<'a>
    
    [<CompiledName ("Run")>]
    val run: DynamicsCont<'a> -> ('a -> unit) -> Dynamics<unit>

    [<CompiledName ("Eta")>]
    val eta: 'a -> DynamicsCont<'a>    
    
    [<CompiledName ("Mu")>]
    val mu: DynamicsCont<DynamicsCont<'a>> -> DynamicsCont<'a>

    [<CompiledName ("Map")>]
    val map: ('a -> 'b) -> DynamicsCont<'a> -> DynamicsCont<'b>

    [<CompiledName ("Ap")>]
    val ap: DynamicsCont<('a -> 'b)> -> DynamicsCont<'a> -> DynamicsCont<'b>
    
    [<CompiledName ("Lift2")>]
    val lift2: ('a -> 'b -> 'c) -> DynamicsCont<'a> -> DynamicsCont<'b> -> DynamicsCont<'c>

    [<CompiledName ("Lift3")>]
    val lift3: ('a -> 'b -> 'c -> 'd) -> DynamicsCont<'a> -> DynamicsCont<'b> -> DynamicsCont<'c> -> DynamicsCont<'d>

    [<CompiledName ("Lift4")>]
    val lift4: ('a -> 'b -> 'c -> 'd -> 'e) -> DynamicsCont<'a> -> DynamicsCont<'b> -> DynamicsCont<'c> -> DynamicsCont<'d> -> DynamicsCont<'e>

    [<CompiledName ("Lift5")>]
    val lift5: ('a -> 'b -> 'c -> 'd -> 'e -> 'f) -> DynamicsCont<'a> -> DynamicsCont<'b> -> DynamicsCont<'c> -> DynamicsCont<'d> -> DynamicsCont<'e> -> DynamicsCont<'f>
    
    [<CompiledName ("Zip")>]
    val zip: DynamicsCont<'a> -> DynamicsCont<'b> -> DynamicsCont<'a * 'b>

    [<CompiledName ("Zip3")>]
    val zip3: DynamicsCont<'a> -> DynamicsCont<'b> -> DynamicsCont<'c> -> DynamicsCont<'a * 'b * 'c>
    