
// SystemDynamics.fsi - Aivika

// Copyright (c) 2009, 2010  David Sorokin. All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 
//   * Redistributions of source code must retain the above copyright
//     notice, this list of conditions and the following disclaimer.
// 
//   * Redistributions in binary form must reproduce the above
//     copyright notice, this list of conditions and the following
//     disclaimer in the documentation and/or other materials
//     provided with the distribution.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
// FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
// THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
// OF THE POSSIBILITY OF SUCH DAMAGE.

//
// Author: David Sorokin <david.sorokin@gmail.com>
//

namespace Maritegra.Aivika

module SD =     
    
    [<CompiledName ("Eta")>]
    val eta: 'a -> Dynamics<'a>

    [<CompiledName ("StartTime")>]
    val starttime: Dynamics<float>

    [<CompiledName ("StopTime")>]
    val stoptime: Dynamics<float>

    [<CompiledName ("DT")>]
    val dt: Dynamics<float>

    [<CompiledName ("Time")>]
    val time: Dynamics<float>
    
    [<CompiledName ("Init")>]
    val init: Dynamics<'a> -> Dynamics<'a>
    
    [<CompiledName ("Discrete")>]
    val discrete: Dynamics<'a> -> Dynamics<'a>
    
    [<CompiledName ("Integ")>]
    val integ: Lazy<Dynamics<float>> -> Dynamics<float> -> Dynamics<float>
    
    [<CompiledName ("Smooth")>]
    val smooth: Dynamics<float> -> Lazy<Dynamics<float>> -> Dynamics<float>
    
    [<CompiledName ("SmoothI")>]
    val smoothI: Lazy<Dynamics<float>> -> Lazy<Dynamics<float>> -> Dynamics<float> -> Dynamics<float>
    
    [<CompiledName ("Smooth3")>]
    val smooth3: Dynamics<float> -> Lazy<Dynamics<float>> -> Dynamics<float>
    
    [<CompiledName ("Smooth3I")>]
    val smooth3I: Lazy<Dynamics<float>> -> Lazy<Dynamics<float>> -> Dynamics<float> -> Dynamics<float>
    
    [<CompiledName ("SmoothN")>]
    val smoothN: Dynamics<float> -> Lazy<Dynamics<float>> -> int -> Dynamics<float>
    
    [<CompiledName ("SmoothNI")>]
    val smoothNI: Lazy<Dynamics<float>> -> Lazy<Dynamics<float>> -> int -> Dynamics<float> -> Dynamics<float>
    
    [<CompiledName ("Delay1")>]
    val delay1: Dynamics<float> -> Dynamics<float> -> Dynamics<float>
    
    [<CompiledName ("Delay1I")>]
    val delay1I: Lazy<Dynamics<float>> -> Dynamics<float> -> Dynamics<float> -> Dynamics<float>
    
    [<CompiledName ("Delay3")>]
    val delay3: Dynamics<float> -> Dynamics<float> -> Dynamics<float>
    
    [<CompiledName ("Delay3I")>]
    val delay3I: Lazy<Dynamics<float>> -> Dynamics<float> -> Dynamics<float> -> Dynamics<float>
    
    [<CompiledName ("DelayN")>]
    val delayN: Dynamics<float> -> Dynamics<float> -> int -> Dynamics<float>
    
    [<CompiledName ("DelayNI")>]
    val delayNI: Lazy<Dynamics<float>> -> Dynamics<float> -> int -> Dynamics<float> -> Dynamics<float>
    
    [<CompiledName ("Forecast")>]
    val forecast: Dynamics<float> -> Dynamics<float> -> Dynamics<float> -> Dynamics<float>
    
    [<CompiledName ("Trend")>]
    val trend: Dynamics<float> -> Dynamics<float> -> Dynamics<float> -> Dynamics<float>
    
    type Reservoir =

        new: init:float -> Reservoir
        new: init:Dynamics<float> -> Reservoir
        
        member Inflow: Dynamics<float> 
        member Inflow: Dynamics<float> with set
        
        member Outflow: Dynamics<float> 
        member Outflow: Dynamics<float> with set
        
        member Value: Dynamics<float>

    type Table =
    
        new: (float * float) [] -> Table 
        
        member LookupF: float -> float
        member LookupStepwiseF: float -> float
        
        member Lookup: Dynamics<float> -> Dynamics<float>
        member LookupStepwise: Dynamics<float> -> Dynamics<float>
        
    [<CompiledName ("Table")>]
    val table: (float * float) [] -> Table

    [<CompiledName ("Lookup")>]
    val lookup: Dynamics<float> -> Table -> Dynamics<float>
    
    [<CompiledName ("LookupF")>]
    val lookupF: float -> Table -> float 
    
    [<CompiledName ("LookupStepwise")>]
    val lookupStepwise: Dynamics<float> -> Table -> Dynamics<float>
    
    [<CompiledName ("LookupStepwiseF")>]
    val lookupStepwiseF: float -> Table -> float 
    
    type Generator = unit -> float
    
    [<CompiledName ("Randomizer")>]
    val randomizer: int option -> (Generator -> Generator) -> Dynamics<Generator -> 'a> -> Dynamics<'a>
    
    [<CompiledName ("Random")>]
    val random: Dynamics<float> -> Dynamics<float> -> Dynamics<float>
    
    [<CompiledName ("RandomS")>]
    val randomS: int -> Dynamics<float> -> Dynamics<float> -> Dynamics<float>

    [<CompiledName ("Normal")>]
    val normal: Dynamics<float> -> Dynamics<float> -> Dynamics<float>
    
    [<CompiledName ("NormalS")>]
    val normalS: int -> Dynamics<float> -> Dynamics<float> -> Dynamics<float>

    [<CompiledName ("Binomial")>]
    val binomial: Dynamics<float> -> Dynamics<int> -> Dynamics<int>

    [<CompiledName ("BinomialS")>]
    val binomialS: int -> Dynamics<float> -> Dynamics<int> -> Dynamics<int>

    [<CompiledName ("Poisson")>]
    val poisson: Dynamics<float> -> Dynamics<int>

    [<CompiledName ("PoissonS")>]
    val poissonS: int -> Dynamics<float> -> Dynamics<int>

    [<CompiledName ("ExpRnd")>]
    val exprnd: Dynamics<float> -> Dynamics<float>

    [<CompiledName ("ExpRndS")>]
    val exprndS: int -> Dynamics<float> -> Dynamics<float>
    
    [<CompiledName ("Delay")>]
    val delay: Dynamics<'a> -> Dynamics<float> -> Dynamics<'a>
    
    [<CompiledName ("DelayI")>]
    val delayI: Lazy<Dynamics<'a>> -> Dynamics<float> -> Dynamics<'a> -> Dynamics<'a>
    
    [<CompiledName ("Step")>]
    val step: Dynamics<float> -> Dynamics<float> -> Dynamics<float>
    
    [<CompiledName ("Pulse")>]
    val pulse: Dynamics<float> -> Dynamics<float> -> Dynamics<float>
    
    [<CompiledName ("PulseP")>]
    val pulseP: Dynamics<float> -> Dynamics<float> -> Dynamics<float> -> Dynamics<float>
    
    [<CompiledName ("Ramp")>]
    val ramp: Dynamics<float> -> Dynamics<float> -> Dynamics<float> -> Dynamics<float>
    
    [<CompiledName ("SinWave")>]
    val sinWave: Dynamics<float> -> Dynamics<float> -> Dynamics<float>
    
    [<CompiledName ("CosWave")>]
    val cosWave: Dynamics<float> -> Dynamics<float> -> Dynamics<float>
    
    [<CompiledName ("MaxD")>]
    val maxD: Dynamics<'a> -> Dynamics<'a> -> Dynamics<'a> when 'a: comparison
    
    [<CompiledName ("MinD")>]
    val minD: Dynamics<'a> -> Dynamics<'a> -> Dynamics<'a> when 'a: comparison
    
    [<CompiledName ("NonNegative")>]
    val nonnegative: Dynamics<float> -> Dynamics<float>
