﻿
(* The Fish Bank model shipped with the installation package of
   Simtegra MapSys [http://www.simtegra.com]. This model uses reservoirs. 
   The commented code is the same model but defined declaratively. *)

#nowarn "40"

open Maritegra.Aivika
open Maritegra.Aivika.SD

module FishBank =

    let specs = {
    
        StartTime=0.0; StopTime=13.0; DT=0.02; 
        Method=RungeKutta4; Randomness=StrongRnd
    }

//    let rec Annual_Profit = Profit
//    and Area = 100.0;
//    and Carrying_Capacity = 1000.0
//    and catch_per_Ship = 
//        table [| (0.0, -0.048); (1.2, 10.875); (2.4, 17.194); 
//                 (3.6, 20.548); (4.8, 22.086); (6.0, 23.344); 
//                 (7.2, 23.903); (8.4, 24.462); (9.6, 24.882); 
//                 (10.8, 25.301); (12.0, 25.86) |]
//        |> lookup Density
//    and Death_Fraction = 
//        table [| (0.0, 5.161); (0.1, 5.161); (0.2, 5.161); 
//                 (0.3, 5.161); (0.4, 5.161); (0.5, 5.161); 
//                 (0.6, 5.118); (0.7, 5.247); (0.8, 5.849); 
//                 (0.9, 6.151); (1.0, 6.194) |]
//        |> lookup (Fish/Carrying_Capacity)
//    and Density = Fish/Area
//    and Fish = integ (lazy (Fish_Hatch_Rate - Fish_Death_Rate - Total_catch_per_Year)) (eta 1000.0)
//    and Fish_Death_Rate = nonnegative (Fish*Death_Fraction)
//    and Fish_Hatch_Rate = nonnegative (Fish*Hatch_Fraction)
//    and Fish_Price = 20.0
//    and Fraction_Invested = 0.2
//    and Hatch_Fraction = 6.0
//    and Operating_Cost = Ships*250.0
//    and Profit = Revenue-Operating_Cost
//    and Revenue = Total_catch_per_Year*Fish_Price
//    and Ship_building_Rate = nonnegative (Profit*Fraction_Invested/Ship_Cost)
//    and Ship_Cost = 300.0
//    and Ships = integ (lazy Ship_building_Rate) (eta 10.0)
//    and Total_catch_per_Year = nonnegative (Ships*catch_per_Ship)
//    and Total_Profit = integ (lazy Annual_Profit) (eta 0.0)

    // 1. Define constants
    
    let Area = 100.0;
    let Carrying_Capacity = 1000.0
    let Fish_Price = 20.0
    let Fraction_Invested = 0.2
    let Hatch_Fraction = 6.0
    let Ship_Cost = 300.0
    
    // 2. Create reservoirs
    
    let Fish_R = Reservoir (1000.0)
    let Ships_R = Reservoir (10.0)
    let Total_Profit_R = Reservoir (0.0)
    
    let Fish = Fish_R.Value
    let Ships = Ships_R.Value
    let Total_Profit = Total_Profit_R.Value
    
    // 3. Define auxiliaries
    
    let Density = Fish/Area
    let catch_per_Ship = 
        table [| (0.0, -0.048); (1.2, 10.875); (2.4, 17.194); 
                 (3.6, 20.548); (4.8, 22.086); (6.0, 23.344); 
                 (7.2, 23.903); (8.4, 24.462); (9.6, 24.882); 
                 (10.8, 25.301); (12.0, 25.86) |]
        |> lookup Density
    let Death_Fraction = 
        table [| (0.0, 5.161); (0.1, 5.161); (0.2, 5.161); 
                 (0.3, 5.161); (0.4, 5.161); (0.5, 5.161); 
                 (0.6, 5.118); (0.7, 5.247); (0.8, 5.849); 
                 (0.9, 6.151); (1.0, 6.194) |]
        |> lookup (Fish/Carrying_Capacity)
    let Fish_Death_Rate = nonnegative (Fish*Death_Fraction)
    let Fish_Hatch_Rate = nonnegative (Fish*Hatch_Fraction)
    let Operating_Cost = Ships*250.0
    let Total_catch_per_Year = nonnegative (Ships*catch_per_Ship)
    let Revenue = Total_catch_per_Year*Fish_Price
    let Profit = Revenue-Operating_Cost
    let Annual_Profit = Profit
    let Ship_building_Rate = nonnegative (Profit*Fraction_Invested/Ship_Cost)
    
    // 4. Define derivatives
    
    Fish_R.Inflow <- Fish_Hatch_Rate - Fish_Death_Rate - Total_catch_per_Year
    Ships_R.Inflow <- Ship_building_Rate
    Total_Profit_R.Inflow <- Annual_Profit
    
    let system = Dynamics.ofArray [| time; Fish; Annual_Profit; Ships |]

    [<EntryPoint>]
    let main args =
    
        let results = Dynamics.run system specs
        
        printfn "Time,Fish,Annual_Profit,Ships"
        
        for r in results do
            printfn "%f,%f,%f,%f" r.[0] r.[1] r.[2] r.[3]
        
        0        